<?php

/**
 * Plugin Name: Sunray Author Manager
 * Plugin URI: http://www.sunraycomputer.com/plugins
 * Description: A versatile plugin for writers to highlight their work.
 * Version: 0.9
 * Author: Matthew Kressel
 * Author URI: http://www.matthewkressel.net
 * License:     GPL2

Sunray Author Manager is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
any later version.

Sunray Author Manager is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Sunray Author Manager. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html

 */


// create new custom type
add_action('init', 'sam_register');

// register admin menu
add_action('admin_menu', 'sam_admin_menu');

// register meta boxes
add_action('add_meta_boxes', 'sam_meta_boxes');

// enqueue datepicker in admin
add_action('admin_enqueue_scripts', 'sam_enqueue_admin');

// enqueue css for users
add_action('wp_enqueue_scripts', 'sam_enqueue');

// save post data
add_action('save_post', 'sam_save');

// filter for posts
add_filter('the_content', 'sam_content');

//add_shortcode('sam_cover_tiles','sam_cover_tiles');
add_shortcode('sam_slider', 'sam_slider');
add_shortcode('sam_biblio', 'sam_biblio');

//add_filter('posts_where', 'sam_posts_where', 10, 2);

// refresh permalinks when loading and unloading
register_deactivation_hook(__FILE__, 'sam_deactivate');
register_activation_hook(__FILE__, 'sam_activate');

register_uninstall_hook(__FILE__, 'sam_deactivate');

// flush rewrite rules on deactivate
function sam_deactivate()
{
    flush_rewrite_rules();
}

// flush rewrite rules on activate
function sam_activate()
{
    sam_register();
    flush_rewrite_rules();
}

// settings page link
function sam_add_settings_link( $links ) {
    $settings_link = '<a href="admin.php?page=sam-options-settings">' . __( 'Settings' ) . '</a>';
    array_push( $links, $settings_link );
    return $links;
}
$plugin = plugin_basename( __FILE__ );
add_filter( "plugin_action_links_$plugin", 'sam_add_settings_link' );


// load scripts
function sam_enqueue_admin($hook)
{
    //wp_enqueue_script('jquery-ui', '//code.jquery.com/ui/1.11.4/jquery-ui.js', array(), '1.0.0', true);
    //wp_enqueue_style('jquery-ui-style', '//code.jquery.com/ui/1.11.4/themes/smoothness/jquery-ui.css');
    wp_enqueue_script( 'jquery-ui-datepicker' );
    //wp_enqueue_style( 'jquery-ui-datepicker' );
    wp_enqueue_style( 'jquery-ui-theme', plugin_dir_url( __FILE__ ) . '/css/jquery-ui.min.css');
    wp_enqueue_script('sam-admin-js', plugin_dir_url(__FILE__) . 'js/sam-admin.js', array('jquery', 'jquery-ui-core', 'jquery-ui-datepicker'));
    wp_enqueue_style('sam-admin-css', plugin_dir_url(__FILE__) . 'css/sam-admin.css');
}

function sam_enqueue($hook)
{
    wp_enqueue_style('slick-css', plugin_dir_url(__FILE__) . 'js/vendor/slick-master/slick/slick.css');
    wp_enqueue_style('slick-theme-css', plugin_dir_url(__FILE__) . 'js/vendor/slick-master/slick/slick-theme.css');

    wp_register_script('slick-js', plugin_dir_url(__FILE__) . 'js/vendor/slick-master/slick/slick.js', array('jquery'));
    wp_enqueue_script('slick-js');

    wp_register_script('sam-user-js', plugin_dir_url(__FILE__) . 'js/sam-user.js', array('jquery'));

    // set speed setting in js
    $sam_options = array(
        'slider_speed' => get_option('sam_slider_speed') ? get_option('sam_slider_speed') : 8
    );
    wp_localize_script( 'sam-user-js', 'sam_options', $sam_options );

    wp_enqueue_script('sam-user-js');

    wp_enqueue_style('sam-user-css', plugin_dir_url(__FILE__) . 'css/sam-user.css');
}


// create custom post types
function sam_register()
{

    global $wp_rewrite;

    // ARTICLES
    $labels = array(
        'name' => _x('Stories', 'post type general name'),
        'singular_name' => _x('Story', 'post type singular name'),
        'add_new' => _x('Add New Story', 'story item'),
        'add_new_item' => __('Add New Story'),
        'edit_item' => __('Edit Story'),
        'new_item' => __('New Story'),
        'view_item' => __('View Story'),
        'search_items' => __('Search Story'),
        'not_found' => __('Nothing found'),
        'not_found_in_trash' => __('Nothing found in Trash'),
        'parent_item_colon' => ''
    );

    $args = array(
        'labels' => $labels,
        'public' => true,
        'publicly_queryable' => true,
        'show_ui' => true,
        'query_var' => true,
        'has_archive' => true,
        'menu_icon' => '',
        'show_in_menu' => 'sam-options',
        'rewrite' => array('slug' => 'stories'),
        'capability_type' => 'post',
        'hierarchical' => false,
        'menu_position' => null,
        'supports' => array('title', 'editor', 'thumbnail', 'excerpt', 'comments', 'revisions')
    );

    register_post_type('sam_story', $args);
    $wp_rewrite->add_permastruct('sam_story', "stories/%sam_story%", true, EP_NONE);

}

// create admin menu item(s)
function sam_admin_menu()
{
    //remove_meta_box('pageparentdiv', 'bcs_story', 'normal');
    add_menu_page('Author Manager', 'Author Manager', 'edit_posts', 'sam-options', '', '', 100);
}

// create meta boxes for post type(s)
function sam_meta_boxes()
{
    add_meta_box('sam-meta', 'Story Information', 'sam_meta_box', 'sam_story', 'normal', 'high');
}

// create publication date meta box
function sam_meta_box($post)
{

    $html = '';

    $title = get_the_title();
    $reprint = get_post_meta($post->ID, 'sam_reprint', true);
    ($reprint == '1') ? $checked = 'checked' : $checked = '';

    $html .= '<div class="sam_title_meta sam_input">';
    $html .= 'Story title (same as Post Title)<br><input readonly type="text" size="75" class="sam_title" name="sam_title" value="' . $title . '"/>&nbsp;';
    $html .= '<label for="sam_reprint" class="selectit"><input type="checkbox" ' . $checked . ' class="sam_reprint" name="sam_reprint" value="1">Reprint</label>';
    $html .= '</div>';

    $byline = get_post_meta($post->ID, 'sam_byline', true);

    /*if(empty($byline)) {
        $current_user = wp_get_current_user();
        $byline = $current_user->display_name;
    }*/

    $html .= '<div class="sam_byline_meta sam_input">';
    $html .= 'Author byline<br><input type="text" size="75" class="sam_byline" name="sam_byline" value="' . $byline . '"/>';
    $html .= '</div>';

    $pub = get_post_meta($post->ID, 'sam_pub_name', true);
    $html .= '<div class="sam_pub_name_meta sam_input">';
    $html .= 'Publication / Venue<br><input type="text" size="75" class="sam_pub_name" name="sam_pub_name" value="' . $pub . '"/>';
    $html .= '</div>';

    $editors = get_post_meta($post->ID, 'sam_editors', true);
    $html .= '<div class="sam_sam_editors_meta sam_input">';
    $html .= 'Editor(s) (separate with commas)<br><input type="text" size="75" class="sam_editors" name="sam_editors" value="' . $editors . '"/>';
    $html .= '</div>';

    $url = get_post_meta($post->ID, 'sam_url', true);
    $html .= '<div class="sam_url_meta sam_input">';
    $html .= 'Link URL<br><input type="text" size="75" class="sam_url" name="sam_url" value="' . $url . '"/>';
    $html .= '</div>';

    $podcast = get_post_meta($post->ID, 'sam_podcast_url', true);
    $html .= '<div class="sam_podcast_url_meta sam_input">';
    $html .= 'Podcast URL<br><input type="text" size="75" class="sam_podcast_url" name="sam_podcast_url" value="' . $podcast . '"/>';
    $html .= '</div>';


    // pub date
    $pub_date = get_post_meta($post->ID, 'sam_pub_date', true);

    if ($pub_date == PHP_INT_MAX) {
        $checked = 'checked';
        $disabled = 'disabled';
        $date = 'Forthcoming';
    } else {
        $checked = '';
        $disabled = '';
        if (empty($pub_date)) $pub_date = time();
        $date = date('F j, Y', $pub_date);
    }


    $html .= '<div class="sam_pub_date_meta sam_input">';
    $html .= 'Publication date<br><input type="text" class="sam_pub_date" ' . $disabled . ' name="sam_pub_date" value="' . $date . '"/>&nbsp;';
    $html .= '<label for="sam_pub_date_forthcoming" class="selectit"><input type="checkbox" ' . $checked . ' class="sam_pub_date_forthcoming" name="sam_pub_date_forthcoming" value="1">Forthcoming</label></p>';
    $html .= '</div>';

    $language = get_post_meta($post->ID, 'sam_language', true);
    $html .= '<div class="sam_language_meta sam_input">';
    $html .= 'Language<br><input type="text" size="75" class="sam_language" name="sam_language" value="' . $language . '"/>';
    $html .= '</div>';

    $translator = get_post_meta($post->ID, 'sam_translator', true);
    $html .= '<div class="sam_translator_meta sam_input">';
    $html .= 'Translator<br><input type="text" size="75" class="sam_translator" name="sam_translator" value="' . $translator . '"/>';
    $html .= '</div>';

    $isbn = get_post_meta($post->ID, 'sam_isbn', true);
    $html .= '<div class="sam_isbn_meta sam_input">';
    $html .= 'ISBN<br><input type="text" size="75" class="sam_isbn" name="sam_isbn" value="' . $isbn . '"/>';
    $html .= '</div>';

    $accolades = get_post_meta($post->ID, 'sam_accolades', true);
    $html .= '<div class="sam_accolades_meta sam_input">';
    $html .= 'Accolades<br><input type="text" size="75" class="sam_accolades" name="sam_accolades" value="' . $accolades . '"/>';
    $html .= '</div>';

    $html .= '<div class="sam_pub_cover_image_meta sam_input">';
    $sam_cover_image = get_post_meta($post->ID, 'sam_cover_image', true);
    $html .= 'Cover image<br>';
    $html .= '<input type="text" size="75" value="' . $sam_cover_image . '" name="sam_cover_image" class="sam_cover_image">&nbsp;';
    $html .= '<input type="button" class="button sam_cover_image_button" value="Browse" name="sam_cover_image_button">';
    $html .= '<p><img src="' . $sam_cover_image . '" class="sam_cover_image_preview"></p></div>';

    $html .= '<input type="hidden" name="sam_meta_noncename" value="' . wp_create_nonce(__FILE__) . '" />';

    echo $html;
}

// save the meta fields
function sam_save($post_id)
{

    if (!isset($_REQUEST['sam_meta_noncename'])) return $post_id;

    // make sure data came from our meta box
    if (!wp_verify_nonce($_REQUEST['sam_meta_noncename'], __FILE__)) return $post_id;

    /* if(isset($_REQUEST['sam_title'])) {
        update_post_meta($post_id, 'sam_title', $_REQUEST['sam_title']);
    } */

    if (isset($_REQUEST['sam_byline'])) {
        $sam_byline = sanitize_text_field($_REQUEST['sam_byline']);
        update_post_meta($post_id, 'sam_byline', $sam_byline);
    }

    if (isset($_REQUEST['sam_url']) and valid_url($_REQUEST['sam_url'])) {
        update_post_meta($post_id, 'sam_url', esc_url($_REQUEST['sam_url']));
    }

    if (isset($_REQUEST['sam_podcast_url']) and valid_url($_REQUEST['sam_podcast_url'])) {
        update_post_meta($post_id, 'sam_podcast_url', esc_url($_REQUEST['sam_podcast_url']));
    }

    if (isset($_REQUEST['sam_pub_name'])) {
        $sam_pub_name = sanitize_text_field($_REQUEST['sam_pub_name']);
        update_post_meta($post_id, 'sam_pub_name', $sam_pub_name);
    }

    if (isset($_REQUEST['sam_editors'])) {
        $sam_editors = sanitize_text_field($_REQUEST['sam_editors']);
        update_post_meta($post_id, 'sam_editors', $sam_editors);
    }

    if (isset($_REQUEST['sam_reprint'])) {
        update_post_meta($post_id, 'sam_reprint', '1');
    } else {
        update_post_meta($post_id, 'sam_reprint', '');
    }

    if (isset($_REQUEST['sam_isbn'])) {
        $sam_isbn = sanitize_text_field($_REQUEST['sam_isbn']);
        update_post_meta($post_id, 'sam_isbn', $sam_isbn);
    }

    if (isset($_REQUEST['sam_pub_date_forthcoming'])) {
        update_post_meta($post_id, 'sam_pub_date', PHP_INT_MAX);
    }

    if (isset($_REQUEST['sam_cover_image']) and valid_url($_REQUEST['sam_cover_image'])) {
        update_post_meta($post_id, 'sam_cover_image', esc_url($_REQUEST['sam_cover_image']));
    }

    if (isset($_REQUEST['sam_pub_date'])) {
        $data = strtotime($_REQUEST['sam_pub_date']);
        update_post_meta($post_id, 'sam_pub_date', $data);
    }

    if (isset($_REQUEST['sam_language'])) {
        $sam_language = sanitize_text_field($_REQUEST['sam_language']);
        update_post_meta($post_id, 'sam_language', $sam_language);
    }

    if (isset($_REQUEST['sam_translator'])) {
        $sam_translator = sanitize_text_field($_REQUEST['sam_translator']);
        update_post_meta($post_id, 'sam_translator', $sam_translator);
    }

    if (isset($_REQUEST['sam_accolades'])) {
        $sam_accolades = sanitize_text_field($_REQUEST['sam_accolades']);
        update_post_meta($post_id, 'sam_accolades', $sam_accolades);
    }

}

function valid_url($url) {
    return (filter_var($url, FILTER_VALIDATE_URL) === true);
}

// add before story
function sam_content($content)
{
    global $post;

    if (is_post_type_archive('sam_story') and in_the_loop()) {
        return sam_do_archive($content);
    }

    if (is_singular('sam_story') and in_the_loop()) {
        return sam_do_single($content);
    }

    return $content;

}

function sam_do_archive($content)
{
    global $post;

    $html = '';

    $sam_cover_image = get_post_meta($post->ID, 'sam_cover_image', true);
    $sam_title = get_the_title();
    $sam_byline = get_post_meta($post->ID, 'sam_byline', true);
    $sam_pub_name = get_post_meta($post->ID, 'sam_pub_name', true);
    $sam_pub_date = get_post_meta($post->ID, 'sam_pub_date', true);
    if($sam_pub_date == PHP_INT_MAX) {
        $sam_pub_date = '<em>Forthcoming</em>';
    } else {
        $sam_pub_date = date('F j, Y', $sam_pub_date);
    }


    $html .= '<div class="sam-story-header">';
    $html .= '<div class="sam-cover-image archive"><a href="' . get_permalink() . '"><img src="' . esc_url($sam_cover_image) . '"></a></div>';
    $html .= '<div class="sam-title sam-meta"><a href="' . get_permalink() . '">&ldquo;' . esc_html($sam_title) . '&rdquo;</a></div>';
    $html .= '<div class="sam-byline sam-meta">By ' . esc_html($sam_byline) . '</div>';
    $html .= '<div class="sam-pub-name sam-meta"><em>' . esc_html($sam_pub_name) . '</em></div>';
    $html .= '<div class="sam-pub-date sam-meta">' . $sam_pub_date . '</div>';
    $html .= '<div class="sam-more-info sam-meta"><a href="' . get_permalink() . '">More information</a></div>';


    $html .= '</div>';

    return $html;
}

// change archive order based on settings

add_action( 'pre_get_posts', 'sam_change_archive_order');
function sam_change_archive_order($query) {

    if(is_admin()) return;

    if(is_archive('sam_stories')) {
        $query->set('order', get_option('sam_biblio_sort_dir'));
        $query->set('orderby', get_option('sam_biblio_sort_by'));
    }

}

function sam_do_single($content)
{
    global $post;

    $html = '';

    $sam_cover_image = get_post_meta($post->ID, 'sam_cover_image', true);
    $sam_title = get_the_title();
    $sam_reprint = get_post_meta($post->ID, 'sam_reprint', true);
    $sam_byline = get_post_meta($post->ID, 'sam_byline', true);
    $sam_pub_name = get_post_meta($post->ID, 'sam_pub_name', true);
    $sam_editors = get_post_meta($post->ID, 'sam_editors', true);
    $sam_language = get_post_meta($post->ID, 'sam_language', true);
    $sam_translator = get_post_meta($post->ID, 'sam_translator', true);
    $sam_accolades = get_post_meta($post->ID, 'sam_accolades', true);
    $sam_pub_date = get_post_meta($post->ID, 'sam_pub_date', true);
    $sam_url = get_post_meta($post->ID, 'sam_url', true);
    $sam_isbn = get_post_meta($post->ID, 'sam_isbn', true);
    $sam_podcast_url = get_post_meta($post->ID, 'sam_podcast_url', true);

    $html .= '<div class="sam-story-header">';

    if (!empty($sam_cover_image)) {
        $html .= '<div class="sam-cover-image"><a href="'.esc_url($sam_cover_image).'"><img src="' . esc_url($sam_cover_image) . '"></a></div>';
    }

    $html .= '<div class="sam-title sam-meta">&ldquo;' . esc_html($sam_title) . '&rdquo;</div>';


    if (!empty($sam_byline)) {
        $html .= '<div class="sam-byline sam-meta">By ' . esc_html($sam_byline) . '</div>';
    }

    if (!empty($sam_pub_name)) {
        $html .= '<div class="sam-pub-name sam-meta"><em>' . esc_html($sam_pub_name) . '</em></div>';
    }

    if (!empty($sam_editors)) {
        if (count(explode(',', $sam_editors)) > 1) {
            $html .= '<div class="sam-editors sam-meta">Eds. ' . esc_html($sam_editors) . '</div>';
        } else {
            $html .= '<div class="sam-editors sam-meta">Ed. ' . esc_html($sam_editors) . '</div>';
        }
    }

    if (!empty($sam_pub_date)) {
        if ($sam_pub_date == PHP_INT_MAX) {
            $html .= '<div class="sam-pub-date sam-meta"><em>Forthcoming</em></div>';
        } else {
            $html .= '<div class="sam-pub-date sam-meta">' . date('F j, Y', $sam_pub_date) . '</div>';
        }
    }

    if (!empty($sam_reprint)) {
        $html .= '<div class="sam-reprint sam-meta">[reprint]</div>';
    }

    if (!empty($sam_language)) {
        $html .= '<div class="sam-language sam-meta">' . esc_html($sam_language) . ' Language</div>';
    }

    if (!empty($sam_translator)) {
        $html .= '<div class="sam-translator sam-meta">Translated by ' . esc_html($sam_translator) . '</div>';
    }

    if (!empty($sam_isbn)) {
        $html .= '<div class="sam-isbn sam-meta">ISBN '.esc_html($sam_isbn).'</div>';
    }

    if (!empty($sam_accolades)) {
        $html .= '<div class="sam-accolades sam-meta"><strong>' . esc_html($sam_accolades) . '</strong></div>';
    }

    if (!empty($sam_url)) {
        $html .= '<div class="sam-url sam-meta"><a target="_blank" href="' . esc_url($sam_url) . '">Link to Story</a></div>';
    }

    if (!empty($sam_podcast_url)) {
        $html .= '<div class="sam-podcast-url sam-meta"><a target="_blank" href="' . esc_url($sam_podcast_url) . '">Link to Podcast</a></div>';
    }

    $html .= '</div>';

    $content = $html . $content;


    return $content;
}


// shortcode function
function sam_biblio($atts)
{

    $opts = shortcode_atts(array(
        'orderby' => get_option('sam_biblio_sort_by'),
        'order' => get_option('sam_biblio_sort_dir'),
        'sort_reprints' => get_option('sam_biblio_sort_reprints'),

    ), $atts);


    // wp_query args
    $args = array('post_type' => array('sam_story'), 'post_status' => 'publish', 'order' => $opts['order'], 'posts_per_page' => -1);

    // sorting options
    switch ($opts['orderby']) {
        case 'title':
            $args = array_merge($args, array('orderby' => 'title'));
            break;
        case 'date':
            $args = array_merge($args, array('meta_key' => 'sam_pub_date', 'orderby' => 'meta_value_num'));
            break;
        default: // default by date
            $args = array_merge($args, array('meta_key' => 'sam_pub_date', 'orderby' => 'meta_value_num'));
            break;
    }

    $my_query = new WP_Query($args);

    if ($my_query->have_posts()) {
        return sam_do_biblio($my_query, $opts);
    }
    wp_reset_postdata();
}

// display the biblio
function sam_do_biblio($my_query, $opts, $nested = false)
{

    $html = '';
    $html .= '<ul class="sam_biblio">';
    while ($my_query->have_posts()) {

        $my_query->the_post();
        $reprint = get_post_meta($my_query->post->ID, 'sam_reprint', true);

        // handle reprints
        if ($nested) {
            if (empty($reprint) and $opts['sort_reprints']) continue;
        } else {
            if (!empty($reprint) and $opts['sort_reprints']) continue;
        }


        $title = get_the_title();
        // $byline = get_post_meta($my_query->post->ID, 'sam_byline', true);
        $pub_name = get_post_meta($my_query->post->ID, 'sam_pub_name', true);
        $editors = get_post_meta($my_query->post->ID, 'sam_editors', true);
        $language = get_post_meta($my_query->post->ID, 'sam_language', true);
        $translator = get_post_meta($my_query->post->ID, 'sam_translator', true);
        $accolades = get_post_meta($my_query->post->ID, 'sam_accolades', true);
        $pub_date = get_post_meta($my_query->post->ID, 'sam_pub_date', true);
        $isbn = get_post_meta($my_query->post->ID, 'sam_isbn', true);


        $html .= '<li>';
        $html .= '&ldquo;' . $title . '.&rdquo; ';
        $html .= '<em>' . esc_html($pub_name) . '</em>. ';
        if (!empty($editors)) {
            (count(explode(',', esc_html($editors))) > 1) ? $eds = 'Eds.' : $eds = 'Ed.';
            $html .= $eds . ' ' . $editors . '. ';
        }
        if (!empty($language)) {
            $html .= esc_html($language) . ' language. ';
        }

        if (!empty($translator)) {
            $html .= 'Translated by ' . esc_html($translator) . '. ';
        }

        if ($pub_date == PHP_INT_MAX) {
            $html .= '<em>Forthcoming</em>. ';
        } else {
            $html .= date('F j, Y', $pub_date) . '. ';
        }
        if (!empty($reprint)) {
            $html .= 'Reprint. ';
        }
        if (!empty($isbn)) {
            $html .= 'ISBN '.esc_html($isbn). '. ';
        }
        if (!empty($accolades)) {
            $html .= '<strong>' . esc_html($accolades) . '</strong>. ';
        }
        $html .= '<a href="' . get_permalink() . '">More info</a>';

        // check for reprints only on first level
        if ($opts['sort_reprints'] and !$nested) {

            /*$args = array(
                'post_type' => 'sam_story',
                'sam_title' => $title,
                'post_status'      => 'publish',
                'orderby'          => 'title',
                'order'            => $opts['order'],
                'posts_per_page'   => -1

            );*/

            $args = array(
                'post_type' => 'sam_story',
                'sam_title' => $title,
                'post_status' => 'publish',
                'meta_key' => 'sam_pub_date',
                'orderby' => 'meta_value_num',
                'order' => 'asc',
                'posts_per_page' => -1

            );


            $nested_query = new WP_Query($args);

            if ($nested_query->have_posts()) {
                $html .= sam_do_biblio($nested_query, $opts, true);
            }

        }

        $html .= '</li>';


    }
    $html .= '</ul>';
    return $html;
}

function sam_posts_where($where, &$wp_query)
{
    global $wpdb;
    if ($sam_title = $wp_query->get('sam_title')) {
        $where .= ' AND ' . $wpdb->posts . '.post_title LIKE \'%' . esc_sql($wpdb->esc_like($sam_title)) . '%\'';
    }
    return $where;
}

function sam_story_columns($columns)
{

    foreach ($columns as $column => $val) {
        $new_columns[$column] = $val;
        if ($column === 'title') {
            $new_columns['sam_pub_name'] = __('Publisher / Venue');
            $new_columns['sam_pub_date'] = __('Publication Date');
        }
    }

    return $new_columns;

}

add_filter('manage_sam_story_posts_columns', 'sam_story_columns');

function sam_story_custom_columns($column)
{
    global $post;

    switch ($column) {
        case 'sam_pub_name':
            $pub_name = get_post_meta($post->ID, 'sam_pub_name', true);
            echo esc_html($pub_name);
            break;
        case 'sam_pub_date':
            $pub_date = get_post_meta($post->ID, 'sam_pub_date', true);
            if ($pub_date == PHP_INT_MAX) {
                echo '<em>Forthcoming</em>';
            } else {
                echo date('F j, Y', $pub_date);
            }
            break;

    }
}

add_action('manage_sam_story_posts_custom_column', 'sam_story_custom_columns');

// Make these columns sortable
function sam_sortable_columns($columns)
{
    $columns['sam_pub_name'] = 'sam_pub_name';
    $columns['sam_pub_date'] = 'sam_pub_date';
    return $columns;
}

add_filter("manage_edit-sam_story_sortable_columns", "sam_sortable_columns");

add_action('pre_get_posts', 'sam_admin_sort_by_date_and_pub_name', 1);
function sam_admin_sort_by_date_and_pub_name($query)
{

   /* if( ! is_admin() )
        return;*/

    /**
     * We only want our code to run in the main WP query
     * AND if an orderby query variable is designated.
     */
    if ($query->is_main_query() && ($orderby = $query->get('orderby'))) {

        switch ($orderby) {
            case 'sam_pub_date':
                $query->set('meta_key', 'sam_pub_date');
                $query->set('orderby', 'meta_value_num');
                break;
            case 'sam_pub_name':
                $query->set('meta_key', 'sam_pub_name');
                $query->set('orderby', 'meta_value');
                break;

        }

    }

}

// create custom plugin settings menu
add_action('admin_menu', 'sam_create_menu');

function sam_create_menu()
{
    //create new top-level menu
    add_submenu_page('sam-options', 'Sunray Author Manager Settings', 'Settings', 'manage_options', 'sam-options-settings', 'sam_settings_page');

    //call register settings function
    add_action('admin_init', 'sam_settings');
}

//register our settings
function sam_settings()
{

    register_setting('sam-settings-group', 'sam_slider_speed');
    register_setting('sam-settings-group', 'sam_slider_sort_by');
    register_setting('sam-settings-group', 'sam_slider_sort_dir');
    register_setting('sam-settings-group', 'sam_slider_height');
    register_setting('sam-settings-group', 'sam_biblio_sort_by');
    register_setting('sam-settings-group', 'sam_biblio_sort_dir');
    register_setting('sam-settings-group', 'sam_biblio_sort_reprints');

}

// settings page
function sam_settings_page()
{
    $slider_speed = get_option('sam_slider_speed') ? get_option('sam_slider_speed') : '8';
    $slider_sort_by = get_option('sam_slider_sort_by') ?  get_option('sam_slider_sort_by') : 'date';
    $slider_sort_dir = get_option('sam_slider_sort_dir') ? get_option('sam_slider_sort_dir') : 'desc';
    $slider_height = get_option('sam_slider_height') ? get_option('sam_slider_height') : '250';

    $biblio_sort_by = get_option('sam_biblio_sort_by') ? get_option('sam_biblio_sort_by') : 'date';
    $biblio_sort_dir = get_option('sam_biblio_sort_dir') ? get_option('sam_biblio_sort_dir') : 'desc';
    $biblio_sort_reprints = get_option('sam_biblio_sort_reprints') ? get_option('sam_biblio_sort_reprints') : '1';


    ?>
    <div class="wrap">
        <h1>Sunray Author Manager Settings</h1>
        <p style="padding: 20px; font-size: 16px; background: #e2e2e2; border: 2px solid: #333;">Created by author Matthew Kressel. <a target="_blank" href="http://www.matthewkressel.net/">Check out my books!</a></p>
        <p>Here you can change the default settings.<p>
        <p><em>Note: these settings will be overridden if shortcode options are provided.</em></p>

        <form method="post" action="options.php">
            <?php settings_fields('sam-settings-group'); ?>
            <?php do_settings_sections('sam-settings-group'); ?>
            <h3>Slider Settings</h3>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Slider Speed (Seconds)</th>
                    <td><input type="text" name="sam_slider_speed"
                               value="<?php echo esc_attr($slider_speed); ?>"/></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Slider Sort By</th>
                    <td>
                    <select name="sam_slider_sort_by">
                        <option value="date">Date</option>
                        <option value="title" <?php echo ($slider_sort_by == 'title') ? 'selected' : ''; ?>>Title</option>
                    </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Slider Sort Direction</th>
                    <td>
                        <select name="sam_slider_sort_dir">
                            <option value="desc">Descending</option>
                            <option value="asc" <?php echo ($slider_sort_dir == 'asc') ? 'selected' : ''; ?>>Ascending</option>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Slider Height (Pixels)</th>
                    <td><input type="text" name="sam_slider_height"
                               value="<?php echo esc_attr($slider_height); ?>"/></td>
                </tr>
            </table>

            <h3>Bibliography Settings</h3>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Bibliography Sort By</th>
                    <td>
                        <select name="sam_biblio_sort_by">
                            <option value="date">Date</option>
                            <option value="title" <?php echo ($biblio_sort_by == 'title') ? 'selected' : ''; ?>>Title</option>
                        </select>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row">Bibliography Sort Direction</th>
                    <td>
                        <select name="sam_biblio_sort_dir">
                            <option value="desc">Descending</option>
                            <option value="asc" <?php echo ($biblio_sort_dir == 'asc') ? 'selected' : ''; ?>>Ascending</option>
                        </select>
                    </td>
                </tr>

                <tr valign="top">
                    <th scope="row">Bibliography Sort Reprints</th>
                    <td>
                        <select name="sam_biblio_sort_reprints">
                            <option value="0">No</option>
                            <option value="1" <?php echo ($biblio_sort_reprints == '1') ? 'selected' : ''; ?>>Yes</option>
                        </select>
                    </td>
                </tr>
            </table>

            <?php submit_button(); ?>

            <p><em>This plugin makes use of the brilliant <a href="http://kenwheeler.github.io/slick/">Slick.js jQuery carousel</a>.</em></p>

        </form>
    </div>
<?php }




// enqueue css for users
//add_action('wp_enqueue_scripts', 'sam_enqueue' );

function sam_slider($atts)
{

    $opts = shortcode_atts(array(
        'orderby' => get_option('sam_slider_sort_by'),
        'order' => get_option('sam_slider_sort_dir'),
    ), $atts);


    // wp_query args
    $args = array('post_type' => array('sam_story'), 'post_status' => 'publish', 'order' => $opts['order'], 'posts_per_page' => -1);

    // sorting options
    switch ($opts['orderby']) {
        case 'title':
            $args = array_merge($args, array('orderby' => 'title'));
            break;
        case 'date':
            $args = array_merge($args, array('meta_key' => 'sam_pub_date', 'orderby' => 'meta_value_num'));
            break;
        default: // default by date
            $args = array_merge($args, array('meta_key' => 'sam_pub_date', 'orderby' => 'meta_value_num'));
            break;
    }

    $my_query = new WP_Query($args);

    if ($my_query->have_posts()) {
        return sam_do_slider($my_query);
    }
    wp_reset_postdata();

}

// display the slider
function sam_do_slider($my_query)
{


    $html = '';
    $html .= '<div class="sam-slider">';
    $slide_height = get_option('sam_slider_height');

    while ($my_query->have_posts()) {
        $my_query->the_post();
        $sam_cover_image = get_post_meta($my_query->post->ID, 'sam_cover_image', true);

        if (empty($sam_cover_image)) continue;

        $pub_name = get_post_meta($my_query->post->ID, 'sam_pub_name', true);

        $html .= '<a class="sam-slide-link" href="' . get_permalink() . '">';
        $html .= '<div class="sam-slide">'; // begin slide
        $html .= '<img class="sam-slide-image" style="height: '.esc_attr($slide_height).'px !important;" src="' . esc_url($sam_cover_image) . '">';

        $html .= '<div class="sam-slide-overlay">'; // begin overlay
        $html .= '<div class="sam-center-text">';
        $html .= '&ldquo;' . get_the_title() . '&rdquo;<br>';
        $html .= '<em>' . esc_html($pub_name) . '</em>';
        $html .= '<div class="sam-more-info">More</div>';
        $html .= '</div>';

        $html .= '</div>'; // end overlay*/
        $html .= '</div>'; // end slide
        $html .= '</a>';

    }
    $html .= '</div>';
    wp_reset_postdata();
    return $html;
}


